import logging
from decimal import Decimal

NUMBER_TYPES = (int, float, Decimal)

__version__ = "1.1.7"
__name__ = "helper-mapilio"
__major__ = False
__minor__ = False
__patch__ = True

logger = logging.getLogger('helper')
